@@ -24,7 +24,6 @@ from typing import (
 
 from attrs import define, field
 
-import cyclopts.utils
 from cyclopts.annotations import resolve_annotated
 from cyclopts.argument import ArgumentCollection
 from cyclopts.bind import create_bound_arguments, is_option_like, normalize_tokens
@@ -41,10 +40,10 @@ from cyclopts.group import Group, sort_groups
 from cyclopts.group_extractors import groups_from_app, inverse_groups_from_app
 from cyclopts.help import (
     HelpPanel,
+    InlineText,
     create_parameter_help_panel,
     format_command_entries,
     format_doc,
-    format_str,
     format_usage,
     resolve_help_format,
     resolve_version_format,
@@ -60,7 +59,7 @@ from cyclopts.utils import (
     to_tuple_converter,
 )
 
-T = TypeVar("T", bound=Callable)
+T = TypeVar("T", bound=Callable[..., Any])
 V = TypeVar("V")
 
 
@@ -102,8 +101,8 @@ def _default_version(default="0.0.0") -> str:
     import importlib
 
     if sys.version_info < (3, 10):  # pragma: no cover
-        from importlib_metadata import PackageNotFoundError
-        from importlib_metadata import version as importlib_metadata_version
+        from importlib_metadata import PackageNotFoundError  # pyright: ignore[reportMissingImports]
+        from importlib_metadata import version as importlib_metadata_version  # pyright: ignore[reportMissingImports]
     else:  # pragma: no cover
         from importlib.metadata import PackageNotFoundError
         from importlib.metadata import version as importlib_metadata_version
@@ -210,11 +209,17 @@ class App:
 
     # Everything below must be kw_only
 
-    default_command: Optional[Callable] = field(default=None, converter=_validate_default_command, kw_only=True)
+    default_command: Optional[Callable[..., Any]] = field(
+        default=None, converter=_validate_default_command, kw_only=True
+    )
     default_parameter: Optional[Parameter] = field(default=None, kw_only=True)
 
     # This can ONLY ever be None or Tuple[Callable, ...]
-    _config: Union[None, Callable, Iterable[Callable]] = field(
+    _config: Union[
+        None,
+        Callable[[list["App"], tuple[str, ...], ArgumentCollection], Any],
+        Iterable[Callable[[list["App"], tuple[str, ...], ArgumentCollection], Any]],
+    ] = field(
         default=None,
         alias="config",
         converter=optional_to_tuple_converter,
@@ -484,7 +489,7 @@ class App:
         if version_raw is None:
             version_raw = "0.0.0"
 
-        version_formatted = format_str(version_raw, format=version_format)
+        version_formatted = InlineText.from_format(version_raw, format=version_format)
         console.print(version_formatted)
 
     @property
@@ -731,10 +736,12 @@ class App:
                 kwargs["group_parameters"] = copy(self._group_parameters)
             if "group_arguments" not in kwargs:
                 kwargs["group_arguments"] = copy(self._group_arguments)
-            app = App(default_command=obj, **kwargs)  # pyright: ignore
+            app = type(self)(**kwargs)  # pyright: ignore
+            # directly call the default decorator, in case we do additional processing there.
+            app.default(obj)
 
-            for flag in chain(kwargs["help_flags"], kwargs["version_flags"]):  # pyright: ignore
-                app[flag].show = False
+        for flag in chain(app.help_flags, app.version_flags):
+            app[flag].show = False
 
         if app._name_transform is None:
             app.name_transform = self.name_transform
@@ -863,7 +870,7 @@ class App:
         *,
         console: Optional["Console"] = None,
         end_of_options_delimiter: Optional[str] = None,
-    ) -> tuple[Callable, inspect.BoundArguments, list[str], dict[str, Any]]:
+    ) -> tuple[Callable[..., Any], inspect.BoundArguments, list[str], dict[str, Any]]:
         """Interpret arguments into a registered function, :class:`~inspect.BoundArguments`, and any remaining unknown tokens.
 
         Parameters
@@ -908,7 +915,7 @@ class App:
         *,
         console: Optional["Console"],
         end_of_options_delimiter: Optional[str],
-    ) -> tuple[Callable, inspect.BoundArguments, list[str], dict[str, Any], ArgumentCollection]:
+    ) -> tuple[Callable[..., Any], inspect.BoundArguments, list[str], dict[str, Any], ArgumentCollection]:
         if tokens is None:
             _log_framework_warning(_detect_test_framework())
 
@@ -943,32 +950,32 @@ class App:
 
         help_flag_index = _get_help_flag_index(tokens, command_app.help_flags)
 
-        if help_flag_index is not None:
-            tokens.pop(help_flag_index)
-
-            help_flag_index = _get_help_flag_index(unused_tokens, command_app.help_flags)
+        try:
             if help_flag_index is not None:
-                unused_tokens.pop(help_flag_index)
+                tokens.pop(help_flag_index)
 
-            if unused_tokens and not command_app.default_command:
-                raise InvalidCommandError(unused_tokens=unused_tokens)
+                help_flag_index = _get_help_flag_index(unused_tokens, command_app.help_flags)
+                if help_flag_index is not None:
+                    unused_tokens.pop(help_flag_index)
 
-            command = self.help_print
-            while meta_parent := meta_parent._meta_parent:
-                command = meta_parent.help_print
-            bound = cyclopts.utils.signature(command).bind(tokens, console=console)
-            unused_tokens = []
-            argument_collection = ArgumentCollection()
-        elif any(flag in tokens for flag in command_app.version_flags):
-            # Version
-            command = self.version_print
-            while meta_parent := meta_parent._meta_parent:
-                command = meta_parent.version_print
-            bound = cyclopts.utils.signature(command).bind()
-            unused_tokens = []
-            argument_collection = ArgumentCollection()
-        else:
-            try:
+                if unused_tokens and not command_app.default_command:
+                    raise InvalidCommandError(unused_tokens=unused_tokens)
+
+                command = self.help_print
+                while meta_parent := meta_parent._meta_parent:
+                    command = meta_parent.help_print
+                bound = inspect.signature(command).bind(tokens, console=console)
+                unused_tokens = []
+                argument_collection = ArgumentCollection()
+            elif any(flag in tokens for flag in command_app.version_flags):
+                # Version
+                command = self.version_print
+                while meta_parent := meta_parent._meta_parent:
+                    command = meta_parent.version_print
+                bound = inspect.signature(command).bind()
+                unused_tokens = []
+                argument_collection = ArgumentCollection()
+            else:
                 if command_app.default_command:
                     command = command_app.default_command
                     validate_command(command)
@@ -1011,17 +1018,17 @@ class App:
                         # Running the application with no arguments and no registered
                         # ``default_command`` will default to ``help_print``.
                         command = self.help_print
-                        bound = cyclopts.utils.signature(command).bind(tokens=tokens, console=console)
+                        bound = inspect.signature(command).bind(tokens=tokens, console=console)
                         unused_tokens = []
                         argument_collection = ArgumentCollection()
-            except CycloptsError as e:
-                e.target = command_app.default_command
-                e.app = command_app
-                if command_chain:
-                    e.command_chain = command_chain
-                if e.console is None:
-                    e.console = self._resolve_console(tokens, console)
-                raise
+        except CycloptsError as e:
+            e.target = command_app.default_command
+            e.app = command_app
+            if command_chain:
+                e.command_chain = command_chain
+            if e.console is None:
+                e.console = self._resolve_console(tokens, console)
+            raise
 
         return command, bound, unused_tokens, ignored, argument_collection
 
@@ -1180,27 +1187,13 @@ class App:
             verbose=verbose,
             end_of_options_delimiter=end_of_options_delimiter,
         )
-        try:
-            if inspect.iscoroutinefunction(command):
-                import asyncio
 
-                return asyncio.run(command(*bound.args, **bound.kwargs))
-            else:
-                return command(*bound.args, **bound.kwargs)
-        except Exception as e:
-            try:
-                from pydantic import ValidationError as PydanticValidationError
-            except ImportError:
-                PydanticValidationError = None  # noqa: N806
+        if inspect.iscoroutinefunction(command):
+            import asyncio
 
-            if PydanticValidationError is not None and isinstance(e, PydanticValidationError):
-                if print_error:
-                    console = self._resolve_console(tokens, console)
-                    console.print(format_cyclopts_error(e))
-
-                if exit_on_error:
-                    sys.exit(1)
-            raise
+            return asyncio.run(command(*bound.args, **bound.kwargs))
+        else:
+            return command(*bound.args, **bound.kwargs)
 
     def _resolve(self, tokens_or_apps: Optional[Sequence], override: Optional[V], attribute: str) -> Optional[V]:
         if override is not None:
@@ -1289,11 +1282,11 @@ class App:
         # Handle commands first; there's an off chance they may be "upgraded"
         # to an argument/parameter panel.
         for subapp in _walk_metas(apps[-1]):
-            # Handle Commands
-            for group, elements in groups_from_app(subapp):
+            for group, subapps in groups_from_app(subapp):
                 if not group.show:
                     continue
 
+                # Fetch a group's help-panel, or create it if it does not yet exist.
                 try:
                     _, command_panel = panels[group.name]
                 except KeyError:
@@ -1304,14 +1297,15 @@ class App:
                     panels[group.name] = (group, command_panel)
 
                 if group.help:
-                    group_help = format_str(group.help, format=help_format)
+                    group_help = InlineText.from_format(group.help, format=help_format, force_empty_end=True)
 
                     if command_panel.description:
                         command_panel.description = RichGroup(command_panel.description, NewLine(), group_help)
                     else:
                         command_panel.description = group_help
 
-                command_panel.entries.extend(format_command_entries(elements, format=help_format))
+                # Add the command to the group's help panel.
+                command_panel.entries.extend(format_command_entries(subapps, format=help_format))
 
         # Handle Arguments/Parameters
         for subapp in _walk_metas(apps[-1]):
