@@ -1,15 +1,25 @@
 import inspect
-from typing import Annotated, Any, ClassVar, Optional, Sequence, get_args, get_origin  # noqa: F401
+import sys
+from typing import (  # noqa: F401
+    Annotated,
+    Any,
+    ClassVar,
+    Optional,
+    Sequence,
+    get_args,
+    get_origin,
+    get_type_hints,
+)
 
 import attrs
 from attrs import field
 
-import cyclopts.utils
 from cyclopts.annotations import (
     NotRequired,
     Required,
     is_annotated,
     is_attrs,
+    is_dataclass,
     is_namedtuple,
     is_pydantic,
     is_typeddict,
@@ -17,6 +27,7 @@ from cyclopts.annotations import (
     resolve_annotated,
     resolve_optional,
 )
+from cyclopts.utils import UNSET
 
 POSITIONAL_OR_KEYWORD = inspect.Parameter.POSITIONAL_OR_KEYWORD
 POSITIONAL_ONLY = inspect.Parameter.POSITIONAL_ONLY
@@ -35,13 +46,16 @@ def _replace_annotated_type(src_type, dst_type):
 class FieldInfo:
     """Extension of :class:`inspect.Parameter`."""
 
-    names: tuple[str, ...]
-    kind: inspect._ParameterKind
+    names: tuple[str, ...] = ()
+    kind: inspect._ParameterKind = inspect.Parameter.POSITIONAL_OR_KEYWORD
 
-    required: bool = field(kw_only=True)
+    required: bool = field(kw_only=True, default=False)
     default: Any = field(default=inspect.Parameter.empty, kw_only=True)
     annotation: Any = field(default=inspect.Parameter.empty, kw_only=True)
 
+    help: Optional[str] = field(default=None, kw_only=True)
+    """Can be populated by additional metadata from another library; e.g. ``pydantic.FieldInfo.description``."""
+
     ###################
     # Class Variables #
     ###################
@@ -51,11 +65,13 @@ class FieldInfo:
     KEYWORD_ONLY: ClassVar = inspect.Parameter.KEYWORD_ONLY
     VAR_POSITIONAL: ClassVar = inspect.Parameter.VAR_POSITIONAL
     VAR_KEYWORD: ClassVar = inspect.Parameter.VAR_KEYWORD
-    POSITIONAL: ClassVar[frozenset] = frozenset({POSITIONAL_OR_KEYWORD, POSITIONAL_ONLY, VAR_POSITIONAL})
-    KEYWORD: ClassVar[frozenset] = frozenset({POSITIONAL_OR_KEYWORD, KEYWORD_ONLY, VAR_KEYWORD})
+    POSITIONAL: ClassVar[frozenset[inspect._ParameterKind]] = frozenset(
+        {POSITIONAL_OR_KEYWORD, POSITIONAL_ONLY, VAR_POSITIONAL}
+    )
+    KEYWORD: ClassVar[frozenset[inspect._ParameterKind]] = frozenset({POSITIONAL_OR_KEYWORD, KEYWORD_ONLY, VAR_KEYWORD})
 
     @classmethod
-    def from_iparam(cls, iparam: inspect.Parameter, *, required: Optional[bool] = None):
+    def from_iparam(cls, iparam: inspect.Parameter, *, annotation: Any = UNSET, required: Optional[bool] = None):
         if required is None:
             required = (
                 iparam.default is iparam.empty
@@ -65,7 +81,7 @@ class FieldInfo:
 
         return cls(
             names=(iparam.name,),
-            annotation=iparam.annotation,
+            annotation=iparam.annotation if annotation is UNSET else annotation,
             kind=iparam.kind,
             default=iparam.default,
             required=required,
@@ -103,12 +119,14 @@ class FieldInfo:
     def is_keyword_only(self) -> bool:
         return self.kind in (KEYWORD_ONLY, VAR_KEYWORD)
 
+    def evolve(self, **kwargs):
+        return attrs.evolve(self, **kwargs)
+
 
 def _typed_dict_field_infos(typeddict) -> dict[str, FieldInfo]:
     # The ``__required_keys__`` and ``__optional_keys__`` attributes of TypedDict are kind of broken in <cp3.11.
     out = {}
-    # Don't use get_type_hints because it resolves Annotated automatically.
-    for name, annotation in typeddict.__annotations__.items():
+    for name, annotation in get_type_hints(typeddict, include_extras=True).items():
         origin = get_origin(resolve_annotated(annotation))
         if origin is Required:
             required = True
@@ -127,16 +145,15 @@ def _generic_class_field_infos(
     include_var_positional=False,
     include_var_keyword=False,
 ) -> dict[str, FieldInfo]:
-    signature = cyclopts.utils.signature(f.__init__)
     out = {}
-    for name, iparam in signature.parameters.items():
-        if iparam.name == "self":
+    for name, field_info in signature_parameters(f.__init__).items():
+        if field_info.name == "self":
             continue
-        if not include_var_positional and iparam.kind is iparam.VAR_POSITIONAL:
+        if not include_var_positional and field_info.kind is field_info.VAR_POSITIONAL:
             continue
-        if not include_var_keyword and iparam.kind is iparam.VAR_KEYWORD:
+        if not include_var_keyword and field_info.kind is field_info.VAR_KEYWORD:
             continue
-        out[name] = FieldInfo.from_iparam(iparam)
+        out[name] = field_info
     return out
 
 
@@ -153,6 +170,12 @@ def _pydantic_field_infos(model) -> dict[str, FieldInfo]:
         else:
             names.append(python_name)
 
+        # Extract Field with description from metadata
+        help = pydantic_field.description or None
+        for meta in pydantic_field.metadata:
+            if hasattr(meta, "description") and meta.description:
+                help = meta.description
+
         # Pydantic places ``Annotated`` data into pydantic.FieldInfo.metadata, while
         # pydantic.FieldInfo.annotation contains the "real" resolved type-hint.
         # We have to re-combine them into a single Annotated hint.
@@ -167,17 +190,19 @@ def _pydantic_field_infos(model) -> dict[str, FieldInfo]:
             annotation=annotation,
             default=FieldInfo.empty if pydantic_field.default is PydanticUndefined else pydantic_field.default,
             required=pydantic_field.is_required(),
+            help=help,
         )
     return out
 
 
 def _namedtuple_field_infos(hint) -> dict[str, FieldInfo]:
     out = {}
+    type_hints = get_type_hints(hint)
     for name in hint._fields:
         out[name] = FieldInfo(
             names=(name,),
             kind=FieldInfo.POSITIONAL_OR_KEYWORD,
-            annotation=hint.__annotations__.get(name, str),
+            annotation=type_hints.get(name, str),
             default=hint._field_defaults.get(name, FieldInfo.empty),
             required=name not in hint._field_defaults,
         )
@@ -186,13 +211,12 @@ def _namedtuple_field_infos(hint) -> dict[str, FieldInfo]:
 
 def _attrs_field_infos(hint) -> dict[str, FieldInfo]:
     out = {}
-    signature = cyclopts.utils.signature(hint.__init__)
-    iparams = signature.parameters
+    field_infos = signature_parameters(hint.__init__)
     for attribute in hint.__attrs_attrs__:
         if not attribute.init:
             continue
 
-        iparam = iparams[attribute.alias]
+        field_info = field_infos[attribute.alias]
 
         if isinstance(attribute.default, attrs.Factory):  # pyright: ignore
             required = False
@@ -204,23 +228,51 @@ def _attrs_field_infos(hint) -> dict[str, FieldInfo]:
             required = False
             default = attribute.default
 
-        out[iparam.name] = FieldInfo(
-            names=(attribute.alias,),
-            annotation=attribute.type,
-            kind=iparam.kind,
-            default=default,
+        out[field_info.name] = field_info.evolve(names=(attribute.alias,), required=required, default=default)
+    return out
+
+
+def _dataclass_field_infos(hint) -> dict[str, FieldInfo]:
+    import dataclasses
+
+    out = {}
+    fields = dataclasses.fields(hint)
+    type_hints = get_type_hints(hint, include_extras=True)  # resolves stringified type hints
+    for f in fields:
+        if f.default_factory is not dataclasses.MISSING:
+            default = f.default_factory()
+            required = False
+        elif f.default is not dataclasses.MISSING:
+            default = f.default
+            required = False
+        else:
+            default = FieldInfo.empty
+            required = True
+
+        annotation = type_hints.get(f.name, FieldInfo.empty)
+
+        if sys.version_info < (3, 10):  # pragma: no cover
+            # Python3.9 does not have Field.kw_only attribute.
+            kind = FieldInfo.POSITIONAL_OR_KEYWORD
+        else:
+            kind = FieldInfo.KEYWORD_ONLY if f.kw_only else FieldInfo.POSITIONAL_OR_KEYWORD
+
+        out[f.name] = FieldInfo(
+            names=(f.name,),
+            kind=kind,
             required=required,
+            annotation=annotation,
+            default=default,
         )
     return out
 
 
-def get_field_infos(
-    hint,
-    *,
-    include_var_positional=False,
-    include_var_keyword=False,
-) -> dict[str, FieldInfo]:
-    if is_pydantic(hint):
+def get_field_infos(hint) -> dict[str, FieldInfo]:
+    if is_dataclass(hint):
+        # This must be before ``is_pydantic`` check so that we
+        # can handle pydantic dataclasses as vanilla dataclasses.
+        return _dataclass_field_infos(hint)
+    elif is_pydantic(hint):
         return _pydantic_field_infos(hint)
     elif is_namedtuple(hint):
         return _namedtuple_field_infos(hint)
@@ -230,3 +282,12 @@ def get_field_infos(
         return _attrs_field_infos(hint)
     else:
         return _generic_class_field_infos(hint)
+
+
+def signature_parameters(f: Any) -> dict[str, FieldInfo]:
+    type_hints = get_type_hints(f, include_extras=True)
+    out = {}
+    for name, iparam in inspect.signature(f).parameters.items():
+        annotation = type_hints.get(name, iparam.annotation)
+        out[name] = FieldInfo.from_iparam(iparam, annotation=annotation)
+    return out
