@@ -1,7 +1,6 @@
 import inspect
 from collections.abc import Iterable
 from copy import deepcopy
-from functools import partial
 from typing import (
     Any,
     Callable,
@@ -21,9 +20,11 @@ from attrs import define, field
 
 import cyclopts._env_var
 import cyclopts.utils
-from cyclopts._convert import ITERABLE_TYPES, convert
+from cyclopts._convert import ITERABLE_TYPES
 from cyclopts.annotations import is_annotated, is_union, resolve_optional
+from cyclopts.field_info import signature_parameters
 from cyclopts.group import Group
+from cyclopts.token import Token
 from cyclopts.utils import (
     default_name_transform,
     frozen,
@@ -106,12 +107,12 @@ class Parameter:
         converter=lambda x: cast(tuple[str, ...], to_tuple_converter(x)),
     )
 
-    _converter: Callable = field(default=None, alias="converter")
+    converter: Optional[Callable[[Any, Sequence[Token]], Any]] = field(default=None)
 
     # This can ONLY ever be a Tuple[Callable, ...]
-    validator: Union[None, Callable, Iterable[Callable]] = field(
+    validator: Union[None, Callable[[Any, Any], Any], Iterable[Callable[[Any, Any], Any]]] = field(
         default=(),
-        converter=lambda x: cast(tuple[Callable, ...], to_tuple_converter(x)),
+        converter=lambda x: cast(tuple[Callable[[Any, Any], Any], ...], to_tuple_converter(x)),
     )
 
     # This can ONLY ever be ``None`` or ``Tuple[str, ...]``
@@ -183,10 +184,6 @@ class Parameter:
     def show(self) -> bool:
         return self._show if self._show is not None else self.parse
 
-    @property
-    def converter(self):
-        return self._converter if self._converter else partial(convert, name_transform=self.name_transform)
-
     @property
     def name_transform(self):
         return self._name_transform if self._name_transform else default_name_transform
@@ -332,15 +329,14 @@ def validate_command(f: Callable):
     """
     if (f.__module__ or "").startswith("cyclopts"):  # Speed optimization.
         return
-    signature = cyclopts.utils.signature(f)
-    for iparam in signature.parameters.values():
+    for field_info in signature_parameters(f).values():
         # Speed optimization: if an object is not annotated, then there's nothing
         # to validate. Checking if there's an annotation is significantly faster
         # than instantiating a cyclopts.Parameter object.
-        if not is_annotated(iparam.annotation):
+        if not is_annotated(field_info.annotation):
             continue
-        _, cparam = Parameter.from_annotation(iparam.annotation)
-        if not cparam.parse and iparam.kind is not iparam.KEYWORD_ONLY:
+        _, cparam = Parameter.from_annotation(field_info.annotation)
+        if not cparam.parse and field_info.kind is not field_info.KEYWORD_ONLY:
             raise ValueError("Parameter.parse=False must be used with a KEYWORD_ONLY function parameter.")
 
 
