@@ -10,8 +10,6 @@ from typing import TYPE_CHECKING, Any, Literal, Optional, Sequence, Tuple, Union
 
 from attrs import field, frozen
 
-# fmt: off
-
 # https://threeofwands.com/attra-iv-zero-overhead-frozen-attrs-classes/
 if TYPE_CHECKING:
     from json import JSONDecodeError
@@ -22,14 +20,6 @@ else:
 
     frozen = functools.partial(define, unsafe_hash=True)
 
-if sys.version_info >= (3, 10):  # pragma: no cover
-    def signature(f: Any) -> inspect.Signature:
-        return inspect.signature(f, eval_str=True)
-else:  # pragma: no cover
-    def signature(f: Any) -> inspect.Signature:
-        return inspect.signature(f)
-# fmt: on
-
 if sys.version_info >= (3, 10):  # pragma: no cover
     from sys import stdlib_module_names
 else:  # pragma: no cover
@@ -279,7 +269,7 @@ def record_init(target: str):
 
     def decorator(cls):
         original_init = cls.__init__
-        function_signature = signature(original_init)
+        function_signature = inspect.signature(original_init)
         param_names = tuple(name for name in function_signature.parameters if name != "self")
 
         @functools.wraps(original_init)
@@ -366,7 +356,7 @@ def default_name_transform(s: str) -> str:
     return s.lower().replace("_", "-").strip("-")
 
 
-def grouper(iterable: Sequence, n: int) -> Iterator[Tuple[Any, ...]]:
+def grouper(iterable: Sequence[Any], n: int) -> Iterator[Tuple[Any, ...]]:
     """Collect data into non-overlapping fixed-length chunks or blocks.
 
     https://docs.python.org/3/library/itertools.html#itertools-recipes
