@@ -87,6 +87,7 @@ def test_help_custom_usage(app, console):
 
 
 def test_help_custom_usage_subapp(app, console):
+    """Intentionally do not print --help/--version flags in subapp help."""
     app.command(App(name="foo", usage="My custom usage."))
 
     with console.capture() as capture:
@@ -97,10 +98,6 @@ def test_help_custom_usage_subapp(app, console):
         """\
         My custom usage.
 
-        ╭─ Commands ─────────────────────────────────────────────────────────╮
-        │ --help -h  Display this message and exit.                          │
-        │ --version  Display application version.                            │
-        ╰────────────────────────────────────────────────────────────────────╯
         """
     )
     assert actual == expected
@@ -1802,3 +1799,38 @@ def test_help_help_on_error(app, console):
         """
     )
     assert actual == expected
+
+
+def test_issue_373_help_space_with_meta_app(app, console):
+    @app.default
+    def default(value: str):
+        print(f"{value=}")
+
+    @app.meta.default
+    def meta(
+        *tokens: Annotated[str, Parameter(show=False, allow_leading_hyphen=True)],
+        meta_value: int = 3,
+    ):
+        app(tokens)
+
+    with console.capture() as capture:
+        app("--help", console=console, exit_on_error=False)
+
+    actual = capture.get()
+    expected = dedent(
+        """\
+        Usage: app COMMAND [ARGS] [OPTIONS]
+
+        App Help String Line 1.
+
+        ╭─ Commands ─────────────────────────────────────────────────────────╮
+        │ --help -h  Display this message and exit.                          │
+        │ --version  Display application version.                            │
+        ╰────────────────────────────────────────────────────────────────────╯
+        ╭─ Parameters ───────────────────────────────────────────────────────╮
+        │ *  VALUE --value  [required]                                       │
+        │    --meta-value   [default: 3]                                     │
+        ╰────────────────────────────────────────────────────────────────────╯
+        """
+    )
+    assert actual == expected
