@@ -1,45 +1,16 @@
 import json
+from dataclasses import dataclass
 from datetime import datetime
 from textwrap import dedent
-from typing import Annotated, Dict, Optional, Union
+from typing import Annotated, Dict, Literal, Optional, Union
 
+import pydantic
 import pytest
 from pydantic import BaseModel, ConfigDict, Field, PositiveInt, validate_call
 from pydantic import ValidationError as PydanticValidationError
 from pydantic.alias_generators import to_camel
 
-from cyclopts import MissingArgumentError, Parameter
-
-
-def test_pydantic_error_msg(app, console):
-    @app.command
-    @validate_call
-    def foo(value: PositiveInt):
-        print(value)
-
-    assert app["foo"].default_command == foo
-
-    foo(1)
-    with pytest.raises(PydanticValidationError):
-        foo(-1)
-
-    with console.capture() as capture, pytest.raises(PydanticValidationError):
-        app(["foo", "-1"], console=console, exit_on_error=False, print_error=True)
-
-    actual = capture.get()
-
-    expected_prefix = dedent(
-        """\
-        ╭─ Error ────────────────────────────────────────────────────────────╮
-        │ 1 validation error for test_pydantic_error_msg.<locals>.foo        │
-        │ 0                                                                  │
-        │   Input should be greater than 0 [type=greater_than,               │
-        │ input_value=-1, input_type=int]                                    │
-        │     For further information visit                                  │
-        """
-    )
-
-    assert actual.startswith(expected_prefix)
+from cyclopts import MissingArgumentError, Parameter, ValidationError
 
 
 # Modified from https://docs.pydantic.dev/latest/#pydantic-examples
@@ -50,13 +21,54 @@ class Outfit(BaseModel):
 
 
 class User(BaseModel):
-    id: int
+    id: PositiveInt
     name: str = Field(default="John Doe")
     signup_ts: Union[datetime, None]
     tastes: Dict[str, PositiveInt]
     outfit: Optional[Outfit] = None
 
 
+def test_pydantic_error_msg(app, console):
+    @app.command
+    @validate_call
+    def foo(value: PositiveInt):
+        print(value)
+
+    assert app["foo"].default_command == foo
+
+    # foo(1)
+    with pytest.raises(PydanticValidationError):
+        foo(-1)
+
+    with console.capture() as capture, pytest.raises(ValidationError):
+        app(["foo", "-1"], console=console, exit_on_error=False, print_error=True)
+
+    actual = capture.get()
+
+    expected_prefix = dedent(
+        """\
+        ╭─ Error ────────────────────────────────────────────────────────────╮
+        │ Invalid value "-1" for "VALUE". 1 validation error for             │
+        │ constrained-int                                                    │
+        │   Input should be greater than 0 [type=greater_than,               │
+        │ input_value=-1, input_type=int]                                    │
+        │     For further information visit                                  │
+        """
+    )
+
+    assert actual.startswith(expected_prefix)
+
+
+def test_pydantic_error_from_function_body(app):
+    @app.command
+    def foo(value: int):
+        # id=-1 is not a valid PositiveError
+        User(id=-1, signup_ts=None, tastes={})
+
+    with pytest.raises(PydanticValidationError):
+        app(["foo", "-1"])
+
+
 def test_bind_pydantic_basemodel(app, assert_parse_args):
     @app.command
     def foo(user: User):
@@ -295,3 +307,161 @@ def test_parameter_decorator_pydantic_nested_1(app, console):
         """
     )
     assert actual == expected
+
+
+def test_pydantic_field_description(app, console):
+    """Test that pydantic Field.description is used for help text."""
+
+    class UserModel(BaseModel):
+        # Simple case - Field.description should be used
+        name: str = Field(description="User name.")
+
+        # Parameter(help=...) takes precedence over Field
+        name_with_param: Annotated[
+            str,
+            Parameter(help="User name with Parameter help."),
+        ] = "Jane Doe"
+
+        # Field description in Annotated should be used
+        name_with_field_in_annotated: Annotated[
+            str,
+            Field(description="Description from Field in Annotated."),
+        ] = "John Doe"
+
+        # Another simple case
+        age: int = Field(description="User age in years.")
+
+    @app.default
+    def main(user: UserModel):
+        pass
+
+    with console.capture() as capture:
+        app("--help", console=console)
+
+    actual = capture.get()
+    # Debugging output removed to avoid cluttering test outputs
+    # If needed, use: logging.debug(f"Actual help content: {actual}")
+
+    # Verify that Field.description is used for help text
+    assert "User name." in actual
+
+    # Verify that Parameter(help=...) takes precedence
+    assert "User name with Parameter help." in actual
+
+    # Verify that Field.description is used from Annotated as well
+    assert "Description from Field in Annotated." in actual
+
+    # Verify the other description is present
+    assert "User age in years." in actual
+
+
+def test_pydantic_annotated_field_discriminator(app, assert_parse_args, console):
+    """From https://github.com/BrianPugh/cyclopts/issues/377"""
+
+    class DatasetImage(pydantic.BaseModel):
+        type: Literal["image"] = "image"
+        path: str
+        resolution: tuple[int, int]
+
+    class DatasetVideo(pydantic.BaseModel):
+        type: Literal["video"] = "video"
+        path: str
+        resolution: tuple[int, int]
+        fps: int
+
+    Dataset = Annotated[Union[DatasetImage, DatasetVideo], pydantic.Field(discriminator="type")]
+
+    @dataclass
+    class Config:
+        dataset: Dataset  # pyright: ignore[reportInvalidTypeForm]
+
+    @app.default
+    def main(
+        config: Annotated[Optional[Config], Parameter(name="*")] = None,
+    ):
+        pass
+
+    assert_parse_args(
+        main,
+        "--dataset.type=image --dataset.path foo.png --dataset.resolution 640 480",
+        Config(DatasetImage(path="foo.png", resolution=(640, 480))),
+    )
+    assert_parse_args(
+        main,
+        "--dataset.type=video --dataset.path foo.mp4 --dataset.resolution 640 480 --dataset.fps 30",
+        Config(DatasetVideo(path="foo.mp4", resolution=(640, 480), fps=30)),
+    )
+
+    with console.capture() as capture:
+        app("--help", console=console)
+
+    actual = capture.get()
+    expected = dedent(
+        """\
+        Usage: main COMMAND [ARGS] [OPTIONS]
+
+        ╭─ Commands ─────────────────────────────────────────────────────────╮
+        │ --help -h  Display this message and exit.                          │
+        │ --version  Display application version.                            │
+        ╰────────────────────────────────────────────────────────────────────╯
+        ╭─ Parameters ───────────────────────────────────────────────────────╮
+        │ DATASET.TYPE               [choices: image, video]                 │
+        │   --dataset.type                                                   │
+        │ DATASET.PATH                                                       │
+        │   --dataset.path                                                   │
+        │ DATASET.RESOLUTION                                                 │
+        │   --dataset.resolution --                                          │
+        │   dataset.empty-resolutio                                          │
+        │   n                                                                │
+        │ DATASET.FPS --dataset.fps                                          │
+        ╰────────────────────────────────────────────────────────────────────╯
+        """
+    )
+    assert actual == expected
+
+
+@pytest.mark.parametrize(
+    "dataclass_decorator",
+    [
+        dataclass,
+        pydantic.dataclasses.dataclass,
+    ],
+)
+def test_pydantic_annotated_field_discriminator_dataclass(app, assert_parse_args, dataclass_decorator):
+    """Pydantic discriminator should work, even if the union'd types are not pydantic.BaseModel."""
+
+    @dataclass_decorator
+    class DatasetImage:
+        type: Literal["image"]
+        path: str
+        resolution: tuple[int, int]
+
+    @dataclass_decorator
+    class DatasetVideo:
+        type: Literal["video"]
+        path: str
+        resolution: tuple[int, int]
+        fps: int
+
+    Dataset = Annotated[Union[DatasetImage, DatasetVideo], pydantic.Field(discriminator="type")]
+
+    @dataclass_decorator
+    class Config:
+        dataset: Dataset  # pyright: ignore[reportInvalidTypeForm]
+
+    @app.default
+    def main(
+        config: Annotated[Optional[Config], Parameter(name="*")] = None,
+    ):
+        pass
+
+    assert_parse_args(
+        main,
+        "--dataset.type=image --dataset.path foo.png --dataset.resolution 640 480",
+        Config(DatasetImage(type="image", path="foo.png", resolution=(640, 480))),  # pyright: ignore
+    )
+    assert_parse_args(
+        main,
+        "--dataset.type=video --dataset.path foo.mp4 --dataset.resolution 640 480 --dataset.fps 30",
+        Config(DatasetVideo(type="video", path="foo.mp4", resolution=(640, 480), fps=30)),  # pyright: ignore
+    )
